@@ -1,14 +1,14 @@
-from __future__ import division
-
-import numpy as np
-import os
 import json
+import os
 from collections import OrderedDict
 
+import numpy as np
+
 from .prior import Prior, PriorDict
-from .utils import (logtrapzexp, check_directory_exists_and_if_not_mkdir,
-                    logger)
-from .utils import BilbyJsonEncoder, load_json, move_old_file
+from .utils import (
+    logtrapzexp, check_directory_exists_and_if_not_mkdir, logger,
+    BilbyJsonEncoder, load_json, move_old_file
+)
 from .result import FileMovedError
 
 
@@ -16,7 +16,7 @@ def grid_file_name(outdir, label, gzip=False):
     """ Returns the standard filename used for a grid file
 
     Parameters
-    ----------
+    ==========
     outdir: str
         Name of the output directory
     label: str
@@ -25,7 +25,7 @@ def grid_file_name(outdir, label, gzip=False):
         Set to True to append `.gz` to the extension for saving in gzipped format
 
     Returns
-    -------
+    =======
     str: File name of the output file
     """
     if gzip:
@@ -41,7 +41,7 @@ class Grid(object):
         """
 
         Parameters
-        ----------
+        ==========
         likelihood: bilby.likelihood.Likelihood
         priors: bilby.prior.PriorDict
         grid_size: int, list, dict
@@ -114,7 +114,7 @@ class Grid(object):
         Marginalize over a list of parameters.
 
         Parameters
-        ----------
+        ==========
         log_array: array_like
             A :class:`numpy.ndarray` of log likelihood/posterior values.
         parameters: list, str
@@ -125,7 +125,7 @@ class Grid(object):
             the set of parameter to *not* marginalize over.
 
         Returns
-        -------
+        =======
         out_array: array_like
             An array containing the marginalized log likelihood/posterior.
         """
@@ -164,7 +164,7 @@ class Grid(object):
         Marginalize the log likelihood/posterior over a single given parameter.
 
         Parameters
-        ----------
+        ==========
         log_array: array_like
             A :class:`numpy.ndarray` of log likelihood/posterior values.
         name: str
@@ -173,7 +173,7 @@ class Grid(object):
             A list of parameter names that have not been marginalized over.
 
         Returns
-        -------
+        =======
         out: array_like
             An array containing the marginalized log likelihood/posterior.
         """
@@ -220,14 +220,14 @@ class Grid(object):
         ln likelihood will be fully marginalized over.
 
         Parameters
-        ----------
+        ==========
         parameters: str, list, optional
             Name of, or list of names of, the parameter(s) to marginalize over.
         not_parameters: str, optional
             Name of, or list of names of, the parameter(s) to not marginalize over.
 
         Returns
-        -------
+        =======
         array-like:
             The marginalized ln likelihood.
         """
@@ -241,14 +241,14 @@ class Grid(object):
         ln posterior will be fully marginalized over.
 
         Parameters
-        ----------
+        ==========
         parameters: str, list, optional
             Name of, or list of names of, the parameter(s) to marginalize over.
         not_parameters: str, optional
             Name of, or list of names of, the parameter(s) to not marginalize over.
 
         Returns
-        -------
+        =======
         array-like:
             The marginalized ln posterior.
         """
@@ -262,14 +262,14 @@ class Grid(object):
         likelihood will be fully marginalized over.
 
         Parameters
-        ----------
+        ==========
         parameters: str, list, optional
             Name of, or list of names of, the parameter(s) to marginalize over.
         not_parameters: str, optional
             Name of, or list of names of, the parameter(s) to not marginalize over.
 
         Returns
-        -------
+        =======
         array-like:
             The marginalized likelihood.
         """
@@ -285,14 +285,14 @@ class Grid(object):
         posterior will be fully marginalized over.
 
         Parameters
-        ----------
+        ==========
         parameters: str, list, optional
             Name of, or list of names of, the parameter(s) to marginalize over.
         not_parameters: str, optional
             Name of, or list of names of, the parameter(s) to not marginalize over.
 
         Returns
-        -------
+        =======
         array-like:
             The marginalized posterior.
         """
@@ -377,7 +377,7 @@ class Grid(object):
         Writes the Grid to a file.
 
         Parameters
-        ----------
+        ==========
         filename: str, optional
             Filename to write to (overwrites the default)
         overwrite: bool, optional
@@ -420,7 +420,7 @@ class Grid(object):
         """ Read in a saved .json grid file
 
         Parameters
-        ----------
+        ==========
         filename: str
             If given, try to load from this filename
         outdir, label: str
@@ -431,11 +431,11 @@ class Grid(object):
             extension)
 
         Returns
-        -------
+        =======
         grid: bilby.core.grid.Grid
 
         Raises
-        -------
+        =======
         ValueError: If no filename is given and either outdir or label is None
                     If no bilby.core.grid.Grid is found in the path
 
