@@ -1,16 +1,14 @@
-from importlib import import_module
-from io import open as ioopen
 import json
 import os
+from importlib import import_module
+from io import open as ioopen
 
-from future.utils import iteritems
-from matplotlib.cbook import flatten
 import numpy as np
 
-from bilby.core.prior.analytical import DeltaFunction
-from bilby.core.prior.base import Prior, Constraint
-from bilby.core.prior.joint import JointPrior
-from bilby.core.utils import logger, check_directory_exists_and_if_not_mkdir, BilbyJsonEncoder, decode_bilby_json
+from .analytical import DeltaFunction
+from .base import Prior, Constraint
+from .joint import JointPrior
+from ..utils import logger, check_directory_exists_and_if_not_mkdir, BilbyJsonEncoder, decode_bilby_json
 
 
 class PriorDict(dict):
@@ -19,7 +17,7 @@ class PriorDict(dict):
         """ A dictionary of priors
 
         Parameters
-        ----------
+        ==========
         dictionary: Union[dict, str, None]
             If given, a dictionary to generate the prior set.
         filename: Union[str, None]
@@ -61,12 +59,12 @@ class PriorDict(dict):
         Placeholder parameter conversion function.
 
         Parameters
-        ----------
+        ==========
         sample: dict
             Dictionary to convert
 
         Returns
-        -------
+        =======
         sample: dict
             Same as input
         """
@@ -76,7 +74,7 @@ class PriorDict(dict):
         """ Write the prior distribution to file.
 
         Parameters
-        ----------
+        ==========
         outdir: str
             output directory name
         label: str
@@ -124,18 +122,20 @@ class PriorDict(dict):
         """ Reads in a prior from a file specification
 
         Parameters
-        ----------
+        ==========
         filename: str
             Name of the file to be read in
 
         Notes
-        -----
+        =====
         Lines beginning with '#' or empty lines will be ignored.
         Priors can be loaded from:
-            bilby.core.prior as, e.g.,    foo = Uniform(minimum=0, maximum=1)
-            floats, e.g.,                 foo = 1
-            bilby.gw.prior as, e.g.,      foo = bilby.gw.prior.AlignedSpin()
-            other external modules, e.g., foo = my.module.CustomPrior(...)
+
+        - bilby.core.prior as, e.g.,    :code:`foo = Uniform(minimum=0, maximum=1)`
+        - floats, e.g.,                 :code:`foo = 1`
+        - bilby.gw.prior as, e.g.,      :code:`foo = bilby.gw.prior.AlignedSpin()`
+        - other external modules, e.g., :code:`foo = my.module.CustomPrior(...)`
+
         """
 
         comments = ['#', '\n']
@@ -154,19 +154,21 @@ class PriorDict(dict):
     @classmethod
     def _get_from_json_dict(cls, prior_dict):
         try:
-            cls == getattr(
+            class_ = getattr(
                 import_module(prior_dict["__module__"]),
                 prior_dict["__name__"])
         except ImportError:
             logger.debug("Cannot import prior module {}.{}".format(
                 prior_dict["__module__"], prior_dict["__name__"]
             ))
+            class_ = cls
         except KeyError:
             logger.debug("Cannot find module name to load")
+            class_ = cls
         for key in ["__module__", "__name__", "__prior_dict__"]:
             if key in prior_dict:
                 del prior_dict[key]
-        obj = cls(dict())
+        obj = class_(dict())
         obj.from_dictionary(prior_dict)
         return obj
 
@@ -175,7 +177,7 @@ class PriorDict(dict):
         """ Reads in a prior from a json file
 
         Parameters
-        ----------
+        ==========
         filename: str
             Name of the file to be read in
         """
@@ -185,7 +187,7 @@ class PriorDict(dict):
 
     def from_dictionary(self, dictionary):
         eval_dict = dict(inf=np.inf)
-        for key, val in iteritems(dictionary):
+        for key, val in dictionary.items():
             if isinstance(val, Prior):
                 continue
             elif isinstance(val, (int, float)):
@@ -206,7 +208,15 @@ class PriorDict(dict):
                     module = __name__.replace(
                         '.' + os.path.basename(__file__).replace('.py', ''), ''
                     )
-                cls = getattr(import_module(module), cls, cls)
+                try:
+                    cls = getattr(import_module(module), cls, cls)
+                except ModuleNotFoundError:
+                    logger.error(
+                        "Cannot import prior class {} for entry: {}={}".format(
+                            cls, key, val
+                        )
+                    )
+                    raise
                 if key.lower() in ["conversion_function", "condition_func"]:
                     setattr(self, key, cls)
                 elif isinstance(cls, str):
@@ -230,9 +240,19 @@ class PriorDict(dict):
                             "= {}. Error message {}".format(key, val, e)
                         )
             elif isinstance(val, dict):
-                logger.warning(
-                    'Cannot convert {} into a prior object. '
-                    'Leaving as dictionary.'.format(key))
+                try:
+                    _class = getattr(
+                        import_module(val.get("__module__", "none")),
+                        val.get("__name__", "none"))
+                    dictionary[key] = _class(**val.get("kwargs", dict()))
+                except ImportError:
+                    logger.debug("Cannot import prior module {}.{}".format(
+                        val.get("__module__", "none"), val.get("__name__", "none")
+                    ))
+                    logger.warning(
+                        'Cannot convert {} into a prior object. '
+                        'Leaving as dictionary.'.format(key))
+                    continue
             else:
                 raise TypeError(
                     "Unable to parse prior, bad entry: {} "
@@ -265,7 +285,7 @@ class PriorDict(dict):
         this will set-up default priors for those as well.
 
         Parameters
-        ----------
+        ==========
         likelihood: bilby.likelihood.GravitationalWaveTransient instance
             Used to infer the set of parameters to fill the prior with
         default_priors_file: str, optional
@@ -273,7 +293,7 @@ class PriorDict(dict):
 
 
         Returns
-        -------
+        =======
         prior: dict
             The filled prior dictionary
 
@@ -302,12 +322,12 @@ class PriorDict(dict):
         """Draw samples from the prior set
 
         Parameters
-        ----------
+        ==========
         size: int or tuple of ints, optional
             See numpy.random.uniform docs
 
         Returns
-        -------
+        =======
         dict: Dictionary of the samples
         """
         return self.sample_subset_constrained(keys=list(self.keys()), size=size)
@@ -316,14 +336,14 @@ class PriorDict(dict):
         """ Return an array of samples
 
         Parameters
-        ----------
+        ==========
         keys: list
             A list of keys to sample in
         size: int
             The number of samples to draw
 
         Returns
-        -------
+        =======
         array: array_like
             An array of shape (len(key), size) of the samples (ordered by keys)
         """
@@ -336,14 +356,14 @@ class PriorDict(dict):
         """Draw samples from the prior set for parameters which are not a DeltaFunction
 
         Parameters
-        ----------
+        ==========
         keys: list
             List of prior keys to draw samples from
         size: int or tuple of ints, optional
             See numpy.random.uniform docs
 
         Returns
-        -------
+        =======
         dict: Dictionary of the drawn samples
         """
         self.convert_floats_to_delta_functions()
@@ -357,6 +377,28 @@ class PriorDict(dict):
                 logger.debug('{} not a known prior.'.format(key))
         return samples
 
+    @property
+    def non_fixed_keys(self):
+        keys = self.keys()
+        keys = [k for k in keys if isinstance(self[k], Prior)]
+        keys = [k for k in keys if self[k].is_fixed is False]
+        keys = [k for k in keys if k not in self.constraint_keys]
+        return keys
+
+    @property
+    def fixed_keys(self):
+        return [
+            k for k, p in self.items()
+            if (p.is_fixed and k not in self.constraint_keys)
+        ]
+
+    @property
+    def constraint_keys(self):
+        return [
+            k for k, p in self.items()
+            if isinstance(p, Constraint)
+        ]
+
     def sample_subset_constrained(self, keys=iter([]), size=None):
         if size is None or size == 1:
             while True:
@@ -391,6 +433,7 @@ class PriorDict(dict):
             samples = self.sample_subset(keys=keys, size=sampling_chunk)
             keep = np.atleast_1d(self.evaluate_constraints(samples))
             if len(keep) == 1:
+                self._cached_normalizations[keys] = 1
                 return 1
             all_samples = {key: np.array([]) for key in keys}
             while np.count_nonzero(keep) < min_accept:
@@ -407,20 +450,23 @@ class PriorDict(dict):
         """
 
         Parameters
-        ----------
+        ==========
         sample: dict
             Dictionary of the samples of which we want to have the probability of
         kwargs:
             The keyword arguments are passed directly to `np.product`
 
         Returns
-        -------
+        =======
         float: Joint probability of all individual sample probabilities
 
         """
         prob = np.product([self[key].prob(sample[key])
                            for key in sample], **kwargs)
 
+        return self.check_prob(sample, prob)
+
+    def check_prob(self, sample, prob):
         ratio = self.normalize_constraint_factor(tuple(sample.keys()))
         if np.all(prob == 0.):
             return prob
@@ -440,21 +486,23 @@ class PriorDict(dict):
         """
 
         Parameters
-        ----------
+        ==========
         sample: dict
             Dictionary of the samples of which to calculate the log probability
         axis: None or int
             Axis along which the summation is performed
 
         Returns
-        -------
+        =======
         float or ndarray:
             Joint log probability of all the individual sample probabilities
 
         """
         ln_prob = np.sum([self[key].ln_prob(sample[key])
                           for key in sample], axis=axis)
+        return self.check_ln_prob(sample, ln_prob)
 
+    def check_ln_prob(self, sample, ln_prob):
         ratio = self.normalize_constraint_factor(tuple(sample.keys()))
         if np.all(np.isinf(ln_prob)):
             return ln_prob
@@ -470,20 +518,36 @@ class PriorDict(dict):
                 constrained_ln_prob[keep] = ln_prob[keep] + np.log(ratio)
                 return constrained_ln_prob
 
+    def cdf(self, sample):
+        """Evaluate the cumulative distribution function at the provided points
+
+        Parameters
+        ----------
+        sample: dict, pandas.DataFrame
+            Dictionary of the samples of which to calculate the CDF
+
+        Returns
+        -------
+        dict, pandas.DataFrame: Dictionary containing the CDF values
+
+        """
+        return sample.__class__({key: self[key].cdf(sample) for key, sample in sample.items()})
+
     def rescale(self, keys, theta):
         """Rescale samples from unit cube to prior
 
         Parameters
-        ----------
+        ==========
         keys: list
             List of prior keys to be rescaled
         theta: list
             List of randomly drawn values on a unit cube associated with the prior keys
 
         Returns
-        -------
+        =======
         list: List of floats containing the rescaled sample
         """
+        from matplotlib.cbook import flatten
         return list(flatten([self[key].rescale(sample) for key, sample in zip(keys, theta)]))
 
     def test_redundancy(self, key, disable_logging=False):
@@ -494,8 +558,8 @@ class PriorDict(dict):
         """
         Test whether there are redundant keys in self.
 
-        Return
-        ------
+        Returns
+        =======
         bool: Whether there are redundancies or not
         """
         redundant = False
@@ -518,14 +582,6 @@ class PriorDict(dict):
         return self.__class__(dictionary=dict(self))
 
 
-class PriorSet(PriorDict):
-
-    def __init__(self, dictionary=None, filename=None):
-        """ DEPRECATED: USE PriorDict INSTEAD"""
-        logger.warning("The name 'PriorSet' is deprecated use 'PriorDict' instead")
-        super(PriorSet, self).__init__(dictionary, filename)
-
-
 class PriorDictException(Exception):
     """ General base class for all prior dict exceptions """
 
@@ -536,7 +592,7 @@ class ConditionalPriorDict(PriorDict):
         """
 
         Parameters
-        ----------
+        ==========
         dictionary: dict
             See parent class
         filename: str
@@ -615,12 +671,12 @@ class ConditionalPriorDict(PriorDict):
         """ Returns the required variables to sample a given conditional key.
 
         Parameters
-        ----------
+        ==========
         key : str
             Name of the key that we want to know the required variables for
 
         Returns
-        ----------
+        ==========
         dict: key/value pairs of the required variables
         """
         return {k: self[k].least_recently_sampled for k in getattr(self[key], 'required_variables', [])}
@@ -629,64 +685,69 @@ class ConditionalPriorDict(PriorDict):
         """
 
         Parameters
-        ----------
+        ==========
         sample: dict
             Dictionary of the samples of which we want to have the probability of
         kwargs:
             The keyword arguments are passed directly to `np.product`
 
         Returns
-        -------
+        =======
         float: Joint probability of all individual sample probabilities
 
         """
-        self._check_resolved()
-        for key, value in sample.items():
-            self[key].least_recently_sampled = value
+        self._prepare_evaluation(*zip(*sample.items()))
         res = [self[key].prob(sample[key], **self.get_required_variables(key)) for key in sample]
-        return np.product(res, **kwargs)
+        prob = np.product(res, **kwargs)
+        return self.check_prob(sample, prob)
 
     def ln_prob(self, sample, axis=None):
         """
 
         Parameters
-        ----------
+        ==========
         sample: dict
             Dictionary of the samples of which we want to have the log probability of
         axis: Union[None, int]
             Axis along which the summation is performed
 
         Returns
-        -------
+        =======
         float: Joint log probability of all the individual sample probabilities
 
         """
-        self._check_resolved()
-        for key, value in sample.items():
-            self[key].least_recently_sampled = value
+        self._prepare_evaluation(*zip(*sample.items()))
         res = [self[key].ln_prob(sample[key], **self.get_required_variables(key)) for key in sample]
-        return np.sum(res, axis=axis)
+        ln_prob = np.sum(res, axis=axis)
+        return self.check_ln_prob(sample, ln_prob)
+
+    def cdf(self, sample):
+        self._prepare_evaluation(*zip(*sample.items()))
+        res = {key: self[key].cdf(sample[key], **self.get_required_variables(key)) for key in sample}
+        return sample.__class__(res)
 
     def rescale(self, keys, theta):
         """Rescale samples from unit cube to prior
 
         Parameters
-        ----------
+        ==========
         keys: list
             List of prior keys to be rescaled
         theta: list
             List of randomly drawn values on a unit cube associated with the prior keys
 
         Returns
-        -------
+        =======
         list: List of floats containing the rescaled sample
         """
+        keys = list(keys)
+        theta = list(theta)
         self._check_resolved()
         self._update_rescale_keys(keys)
         result = dict()
         for key, index in zip(self.sorted_keys_without_fixed_parameters, self._rescale_indexes):
-            required_variables = {k: result[k] for k in getattr(self[key], 'required_variables', [])}
-            result[key] = self[key].rescale(theta[index], **required_variables)
+            result[key] = self[key].rescale(theta[index], **self.get_required_variables(key))
+            self[key].least_recently_sampled = result[key]
         return [result[key] for key in keys]
 
     def _update_rescale_keys(self, keys):
@@ -694,6 +755,11 @@ class ConditionalPriorDict(PriorDict):
             self._rescale_indexes = [keys.index(element) for element in self.sorted_keys_without_fixed_parameters]
             self._least_recently_rescaled_keys = keys
 
+    def _prepare_evaluation(self, keys, theta):
+        self._check_resolved()
+        for key, value in zip(keys, theta):
+            self[key].least_recently_sampled = value
+
     def _check_resolved(self):
         if not self._resolved:
             raise IllegalConditionsException("The current set of priors contains unresolveable conditions.")
@@ -774,14 +840,14 @@ def create_default_prior(name, default_priors_file=None):
     """Make a default prior for a parameter with a known name.
 
     Parameters
-    ----------
+    ==========
     name: str
         Parameter name
     default_priors_file: str, optional
         If given, a file containing the default priors.
 
-    Return
-    ------
+    Returns
+    ======
     prior: Prior
         Default prior distribution for that parameter, if unknown None is
         returned.
