@@ -1,19 +1,15 @@
-from __future__ import absolute_import, print_function
-
-from collections import namedtuple
 import os
 import signal
 import shutil
-from shutil import copyfile
 import sys
+from collections import namedtuple
+from distutils.version import LooseVersion
+from shutil import copyfile
 
 import numpy as np
 from pandas import DataFrame
-from distutils.version import LooseVersion
-import dill as pickle
 
-from ..utils import (
-    logger, get_progress_bar, check_directory_exists_and_if_not_mkdir)
+from ..utils import logger, check_directory_exists_and_if_not_mkdir
 from .base_sampler import MCMCSampler, SamplerError
 
 
@@ -25,8 +21,8 @@ class Emcee(MCMCSampler):
     documentation for that class for further help. Under Other Parameters, we
     list commonly used kwargs and the bilby defaults.
 
-    Other Parameters
-    ----------------
+    Parameters
+    ==========
     nwalkers: int, (100)
         The number of walkers
     nsteps: int, (100)
@@ -44,7 +40,6 @@ class Emcee(MCMCSampler):
     a: float (2)
         The proposal scale factor
 
-
     """
 
     default_kwargs = dict(
@@ -224,7 +219,7 @@ class Emcee(MCMCSampler):
         """ Defines various things related to checkpointing and storing data
 
         Returns
-        -------
+        =======
         checkpoint_info: named_tuple
             An object with attributes `sampler_file`, `chain_file`, and
             `chain_template`. The first two give paths to where the sampler and
@@ -258,13 +253,14 @@ class Emcee(MCMCSampler):
 
     def checkpoint(self):
         """ Writes a pickle file of the sampler to disk using dill """
+        import dill
         logger.info("Checkpointing sampler to file {}"
                     .format(self.checkpoint_info.sampler_file))
         with open(self.checkpoint_info.sampler_file, 'wb') as f:
             # Overwrites the stored sampler chain with one that is truncated
             # to only the completed steps
             self.sampler._chain = self.sampler_chain
-            pickle.dump(self._sampler, f)
+            dill.dump(self._sampler, f)
 
     def checkpoint_and_exit(self, signum, frame):
         logger.info("Recieved signal {}".format(signum))
@@ -287,10 +283,11 @@ class Emcee(MCMCSampler):
         if hasattr(self, '_sampler'):
             pass
         elif self.resume and os.path.isfile(self.checkpoint_info.sampler_file):
+            import dill
             logger.info("Resuming run from checkpoint file {}"
                         .format(self.checkpoint_info.sampler_file))
             with open(self.checkpoint_info.sampler_file, 'rb') as f:
-                self._sampler = pickle.load(f)
+                self._sampler = dill.load(f)
             self._set_pos0_for_resume()
         else:
             self._initialise_sampler()
@@ -353,7 +350,7 @@ class Emcee(MCMCSampler):
         self.pos0 = self.sampler.chain[:, -1, :]
 
     def run_sampler(self):
-        tqdm = get_progress_bar()
+        from tqdm.auto import tqdm
         sampler_function_kwargs = self.sampler_function_kwargs
         iterations = sampler_function_kwargs.pop('iterations')
         iterations -= self._previous_iterations
