@@ -1,16 +1,13 @@
 import os
 import sys
-import warnings
 
 import numpy as np
 import math
 
 from ...core import utils
 from ...core.utils import logger
-from .. import utils as gwutils
 from .interferometer import Interferometer
 from .psd import PowerSpectralDensity
-from .strain_data import InterferometerStrainData
 
 
 class InterferometerList(list):
@@ -23,7 +20,7 @@ class InterferometerList(list):
         object has the data used in evaluating the likelihood
 
         Parameters
-        ----------
+        ==========
         interferometers: iterable
             The list of interferometers
         """
@@ -76,7 +73,7 @@ class InterferometerList(list):
         `bilby.gw.detector.InterferometerStrainData` for further information.
 
         Parameters
-        ----------
+        ==========
         sampling_frequency: float
             The sampling frequency (in Hz)
         duration: float
@@ -98,7 +95,7 @@ class InterferometerList(list):
         `bilby.gw.detector.InterferometerStrainData` for further information.
 
         Parameters
-        ----------
+        ==========
         sampling_frequency: float
             The sampling frequency (in Hz)
         duration: float
@@ -116,7 +113,7 @@ class InterferometerList(list):
         """ Inject a signal into noise in each of the three detectors.
 
         Parameters
-        ----------
+        ==========
         parameters: dict
             Parameters of the injection.
         injection_polarizations: dict
@@ -128,14 +125,14 @@ class InterferometerList(list):
             A WaveformGenerator instance using the source model to inject. If
             `injection_polarizations` is given, this will be ignored.
 
-        Note
-        ----------
+        Notes
+        ==========
         if your signal takes a substantial amount of time to generate, or
         you experience buggy behaviour. It is preferable to provide the
         injection_polarizations directly.
 
         Returns
-        -------
+        =======
         injection_polarizations: dict
 
         """
@@ -159,7 +156,7 @@ class InterferometerList(list):
         """ Creates a save file for the data in plain text format
 
         Parameters
-        ----------
+        ==========
         outdir: str
             The output directory in which the data is supposed to be saved
         label: str
@@ -217,38 +214,47 @@ class InterferometerList(list):
                 for interferometer in self}
 
     @staticmethod
-    def _hdf5_filename_from_outdir_label(outdir, label):
-        return os.path.join(outdir, label + '.h5')
+    def _filename_from_outdir_label_extension(outdir, label, extension="h5"):
+        return os.path.join(outdir, label + f'.{extension}')
+
+    _save_docstring = """ Saves the object to a {format} file
+
+    {extra}
+
+    Parameters
+    ==========
+    outdir: str, optional
+        Output directory name of the file
+    label: str, optional
+        Output file name, is 'ifo_list' if not given otherwise. A list of
+        the included interferometers will be appended.
+    """
+
+    _load_docstring = """ Loads in an InterferometerList object from a {format} file
+
+    Parameters
+    ==========
+    filename: str
+        If given, try to load from this filename
+
+    """
 
     def to_hdf5(self, outdir='outdir', label='ifo_list'):
-        """ Saves the object to a hdf5 file
-
-        Parameters
-        ----------
-        outdir: str, optional
-            Output directory name of the file
-        label: str, optional
-            Output file name, is 'ifo_list' if not given otherwise. A list of
-            the included interferometers will be appended.
-        """
         import deepdish
         if sys.version_info[0] < 3:
             raise NotImplementedError('Pickling of InterferometerList is not supported in Python 2.'
                                       'Use Python 3 instead.')
         label = label + '_' + ''.join(ifo.name for ifo in self)
         utils.check_directory_exists_and_if_not_mkdir(outdir)
-        deepdish.io.save(self._hdf5_filename_from_outdir_label(outdir, label), self)
+        try:
+            filename = self._filename_from_outdir_label_extension(outdir, label, "h5")
+            deepdish.io.save(filename, self)
+        except AttributeError:
+            logger.warning("Saving to hdf5 using deepdish failed. Pickle dumping instead.")
+            self.to_pickle(outdir=outdir, label=label)
 
     @classmethod
     def from_hdf5(cls, filename=None):
-        """ Loads in an InterferometerList object from an hdf5 file
-
-        Parameters
-        ----------
-        filename: str
-            If given, try to load from this filename
-
-        """
         import deepdish
         if sys.version_info[0] < 3:
             raise NotImplementedError('Pickling of InterferometerList is not supported in Python 2.'
@@ -260,6 +266,33 @@ class InterferometerList(list):
             raise TypeError('The loaded object is not a InterferometerList')
         return res
 
+    def to_pickle(self, outdir="outdir", label="ifo_list"):
+        import dill
+        utils.check_directory_exists_and_if_not_mkdir('outdir')
+        label = label + '_' + ''.join(ifo.name for ifo in self)
+        filename = self._filename_from_outdir_label_extension(outdir, label, extension="pkl")
+        with open(filename, "wb") as ff:
+            dill.dump(self, ff)
+
+    @classmethod
+    def from_pickle(cls, filename=None):
+        import dill
+        with open(filename, "rb") as ff:
+            res = dill.load(ff)
+        if res.__class__ != cls:
+            raise TypeError('The loaded object is not an InterferometerList')
+        return res
+
+    to_hdf5.__doc__ = _save_docstring.format(
+        format="hdf5", extra=""".. deprecated:: 1.1.0
+    Use :func:`to_pickle` instead."""
+    )
+    to_pickle.__doc__ = _save_docstring.format(
+        format="pickle", extra=".. versionadded:: 1.1.0"
+    )
+    from_hdf5.__doc__ = _load_docstring.format(format="hdf5")
+    from_pickle.__doc__ = _load_docstring.format(format="pickle")
+
 
 class TriangularInterferometer(InterferometerList):
 
@@ -288,73 +321,6 @@ class TriangularInterferometer(InterferometerList):
             longitude += np.arctan(length * np.cos(xarm_azimuth * np.pi / 180) * 1e3 / utils.radius_of_earth)
 
 
-def get_event_data(
-        event, interferometer_names=None, duration=4, roll_off=0.2,
-        psd_offset=-1024, psd_duration=100, cache=True, outdir='outdir',
-        label=None, plot=True, filter_freq=None, **kwargs):
-    """
-    Get open data for a specified event.
-
-    Parameters
-    ----------
-    event: str
-        Event descriptor, this can deal with some prefixes, e.g., '150914',
-        'GW150914', 'LVT151012'
-    interferometer_names: list, optional
-        List of interferometer identifiers, e.g., 'H1'.
-        If None will look for data in 'H1', 'V1', 'L1'
-    duration: float
-        Time duration to search for.
-    roll_off: float
-        The roll-off (in seconds) used in the Tukey window.
-    psd_offset, psd_duration: float
-        The power spectral density (psd) is estimated using data from
-        `center_time+psd_offset` to `center_time+psd_offset + psd_duration`.
-    cache: bool
-        Whether or not to store the acquired data.
-    outdir: str
-        Directory where the psd files are saved
-    label: str
-        If given, an identifying label used in generating file names.
-    plot: bool
-        If true, create an ASD + strain plot
-    filter_freq: float
-        Low pass filter frequency
-    **kwargs:
-        All keyword arguments are passed to
-        `gwpy.timeseries.TimeSeries.fetch_open_data()`.
-
-    Return
-    ------
-    list: A list of bilby.gw.detector.Interferometer objects
-    """
-
-    warnings.warn(
-        "get_event_data is deprecated, use set_strain_data_from_gwpy instead",
-        DeprecationWarning
-    )
-
-    event_time = gwutils.get_event_time(event)
-
-    interferometers = []
-
-    if interferometer_names is None:
-        interferometer_names = ['H1', 'L1', 'V1']
-
-    for name in interferometer_names:
-        try:
-            interferometers.append(get_interferometer_with_open_data(
-                name, trigger_time=event_time, duration=duration, roll_off=roll_off,
-                psd_offset=psd_offset, psd_duration=psd_duration, cache=cache,
-                outdir=outdir, label=label, plot=plot, filter_freq=filter_freq,
-                **kwargs))
-        except ValueError as e:
-            logger.debug("Error raised {}".format(e))
-            logger.warning('No data found for {}.'.format(name))
-
-    return InterferometerList(interferometers)
-
-
 def get_empty_interferometer(name):
     """
     Get an interferometer with standard parameters for known detectors.
@@ -376,12 +342,12 @@ def get_empty_interferometer(name):
 
 
     Parameters
-    ----------
+    ==========
     name: str
         Interferometer identifier.
 
     Returns
-    -------
+    =======
     interferometer: Interferometer
         Interferometer instance
     """
@@ -416,90 +382,3 @@ def load_interferometer(filename):
     else:
         raise IOError("{} could not be loaded. Invalid parameter 'shape'.".format(filename))
     return ifo
-
-
-def get_interferometer_with_open_data(
-        name, trigger_time, duration=4, start_time=None, roll_off=0.2,
-        psd_offset=-1024, psd_duration=100, cache=True, outdir='outdir',
-        label=None, plot=True, filter_freq=None, **kwargs):
-    """
-    Helper function to obtain an Interferometer instance with appropriate
-    PSD and data, given an center_time.
-
-    Parameters
-    ----------
-
-    name: str
-        Detector name, e.g., 'H1'.
-    trigger_time: float
-        Trigger GPS time.
-    duration: float, optional
-        The total time (in seconds) to analyse. Defaults to 4s.
-    start_time: float, optional
-        Beginning of the segment, if None, the trigger is placed 2s before the end
-        of the segment.
-    roll_off: float
-        The roll-off (in seconds) used in the Tukey window.
-    psd_offset, psd_duration: float
-        The power spectral density (psd) is estimated using data from
-        `center_time+psd_offset` to `center_time+psd_offset + psd_duration`.
-    cache: bool, optional
-        Whether or not to store the acquired data
-    outdir: str
-        Directory where the psd files are saved
-    label: str
-        If given, an identifying label used in generating file names.
-    plot: bool
-        If true, create an ASD + strain plot
-    filter_freq: float
-        Low pass filter frequency
-    **kwargs:
-        All keyword arguments are passed to
-        `gwpy.timeseries.TimeSeries.fetch_open_data()`.
-
-    Returns
-    -------
-    bilby.gw.detector.Interferometer: An Interferometer instance with a PSD and frequency-domain strain data.
-
-    """
-
-    warnings.warn(
-        "get_interferometer_with_open_data is deprecated, use set_strain_data_from_gwpy instead",
-        DeprecationWarning
-    )
-
-    logger.warning(
-        "Parameter estimation for real interferometer data in bilby is in "
-        "alpha testing at the moment: the routines for windowing and filtering"
-        " have not been reviewed.")
-
-    utils.check_directory_exists_and_if_not_mkdir(outdir)
-
-    if start_time is None:
-        start_time = trigger_time + 2 - duration
-
-    strain = InterferometerStrainData(roll_off=roll_off)
-    strain.set_from_open_data(
-        name=name, start_time=start_time, duration=duration,
-        outdir=outdir, cache=cache, **kwargs)
-    strain.low_pass_filter(filter_freq)
-
-    strain_psd = InterferometerStrainData(roll_off=roll_off)
-    strain_psd.set_from_open_data(
-        name=name, start_time=start_time + duration + psd_offset,
-        duration=psd_duration, outdir=outdir, cache=cache, **kwargs)
-    # Low pass filter
-    strain_psd.low_pass_filter(filter_freq)
-    # Create and save PSDs
-    psd_frequencies, psd_array = strain_psd.create_power_spectral_density(
-        name=name, outdir=outdir, fft_length=strain.duration)
-
-    interferometer = get_empty_interferometer(name)
-    interferometer.power_spectral_density = PowerSpectralDensity(
-        psd_array=psd_array, frequency_array=psd_frequencies)
-    interferometer.strain_data = strain
-
-    if plot:
-        interferometer.plot_data(outdir=outdir, label=label)
-
-    return interferometer
