@@ -41,11 +41,6 @@ class TestPriorDict(unittest.TestCase):
         priors = bilby.core.prior.PriorDict(self.priors)
         self.assertEqual(priors, priors.copy())
 
-    def test_prior_set(self):
-        priors_dict = bilby.core.prior.PriorDict(self.priors)
-        priors_set = bilby.core.prior.PriorSet(self.priors)
-        self.assertEqual(priors_dict, priors_set)
-
     def test_prior_set_is_dict(self):
         self.assertIsInstance(self.prior_set_from_dict, dict)
 
@@ -303,6 +298,20 @@ class TestPriorDict(unittest.TestCase):
             ),
         )
 
+    def test_cdf(self):
+        """
+        Test that the CDF method is the inverse of the rescale method.
+
+        Note that the format of inputs/outputs is different between the two methods.
+        """
+        sample = self.prior_set_from_dict.sample()
+        self.assertEqual(
+            self.prior_set_from_dict.rescale(
+                sample.keys(),
+                self.prior_set_from_dict.cdf(sample=sample).values()
+            ), list(sample.values())
+        )
+
     def test_redundancy(self):
         for key in self.prior_set_from_dict.keys():
             self.assertFalse(self.prior_set_from_dict.test_redundancy(key=key))
