@@ -164,12 +164,13 @@ class TestGWTransient(unittest.TestCase):
         expected = (
             "GravitationalWaveTransient(interferometers={},\n\twaveform_generator={},\n\t"
             "time_marginalization={}, distance_marginalization={}, phase_marginalization={}, "
-            "priors={})".format(
+            "calibration_marginalization={}, priors={})".format(
                 self.interferometers,
                 self.waveform_generator,
                 False,
                 False,
                 False,
+                False,
                 self.prior,
             )
         )
@@ -211,6 +212,7 @@ class TestGWTransient(unittest.TestCase):
             time_marginalization=False,
             phase_marginalization=False,
             distance_marginalization=False,
+            calibration_marginalization=False,
             waveform_generator_class=self.waveform_generator.__class__,
             waveform_arguments=self.waveform_generator.waveform_arguments,
             frequency_domain_source_model=self.waveform_generator.frequency_domain_source_model,
@@ -828,7 +830,7 @@ class TestROQLikelihood(unittest.TestCase):
         roq_wfg = bilby.gw.waveform_generator.WaveformGenerator(
             duration=self.duration,
             sampling_frequency=self.sampling_frequency,
-            frequency_domain_source_model=bilby.gw.source.roq,
+            frequency_domain_source_model=bilby.gw.source.binary_black_hole_roq,
             waveform_arguments=dict(
                 frequency_nodes_linear=fnodes_linear,
                 frequency_nodes_quadratic=fnodes_quadratic,
@@ -1078,7 +1080,7 @@ class TestRescaledROQLikelihood(unittest.TestCase):
         self.roq_wfg = bilby.gw.waveform_generator.WaveformGenerator(
             duration=self.duration,
             sampling_frequency=self.sampling_frequency,
-            frequency_domain_source_model=bilby.gw.source.roq,
+            frequency_domain_source_model=bilby.gw.source.binary_black_hole_roq,
             waveform_arguments=dict(
                 frequency_nodes_linear=fnodes_linear,
                 frequency_nodes_quadratic=fnodes_quadratic,
